#!/usr/bin/env python
from gimpfu import *
import decimal

'''	 
  BA_FakeEmboss_v1b
  
  By Brian Andersen (animicule@gimpchat.com)
  March 12, 2017, Sept 18, 2017.
  Filter uses GMIC for 1.7.9 only.
  Updated for GMIC 2.0.0 QT, 
  Updated and tested working in GMIC 2.0.4QT Sept 18, 2017
  Some presets are included for entertainment.
  Filter developed on Windows 7 64bit and Gimp Version 2.8.16
  Filter tested and working in Gimp up to Version 2.8.22
 ...................................................... 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 You can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 ___________________________________________________________________

 -------------------------------------------------------------------
'''


def BA_FakeEmboss_v1b (inImage, inDrawable, intensity, doublesize, targetwidth ): 
    GmicVer = 1
    pdb.gimp_image_undo_group_start(inImage)
    pdb.gimp_context_push()
    pdb.gimp_context_set_interpolation(INTERPOLATION_NOHALO) 
    
    width      = inImage.width
    height    = inImage.height
    type   = inImage.base_type
    intensity = int(intensity)
    targetw = int(targetwidth)
    TargetW = 0   

    if intensity == 0:
        suffix1= "Low"
    if intensity == 1:
        suffix1= "Med"
    if intensity == 2:
        suffix1="High"

# create name string for file export 
    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = "_BA_FakeEmboss_"+suffix1

# increments of 250 for targetwidth choice 
    if targetw > 0:
        TargetW = targetw*250
    if doublesize:
        TargetW = TargetW * 2
    if TargetW == 0:
        scale = 1
    else:
        scale = decimal.Decimal(TargetW)/decimal.Decimal(width)
    
    processwidth =int(width *scale)
    processheight=int(height*scale)
       
# scale, create subject layer
    newImage = pdb.gimp_image_new (processwidth, processheight, type)
    origBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage) 
    origBG.mode = LAYER_MODE_NORMAL_LEGACY
    origBG.name = "Original Subject"
    origBG.opacity = 100.0 
    newImage.add_layer (origBG, 0)
    pdb.gimp_item_set_visible(origBG, False) # visiblity off for merge visible   
    pdb.gimp_layer_scale(origBG, processwidth, processheight, False)
    alpha=pdb.gimp_drawable_has_alpha(origBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (origBG)    #  RGBA
      
# dup subject layer to top with name 'temp convolution' set visible for merge visible
    tempoLay = origBG.copy()
    tempoLay.mode = LAYER_MODE_NORMAL_LEGACY
    tempoLay.name = "Temp Convolution"
    tempoLay.opacity = 100.0 
    newImage.add_layer(tempoLay, 0)
    pdb.gimp_item_set_visible(tempoLay, True)   

# dup subject layer to top, smooth bilateral, set visible for merge visible
    bilatLay = origBG.copy()
    bilatLay.mode = LAYER_MODE_NORMAL_LEGACY
    bilatLay.name = "Temp BiLateral"
    bilatLay.opacity = 100.0 
    newImage.add_layer(bilatLay, 0)
    pdb.gimp_item_set_visible(bilatLay, True)
    # [G'MIC] Smooth [bilateral]: -gimp_smooth_bilateral 10,12,2,0,0
    if GmicVer == 0:
        pdb.plug_in_gmic(newImage, bilatLay, 1,    # 1=input only active layer
        "-v - -gimp_smooth_bilateral 10,12,2,0,0")
    elif GmicVer == 1:
        pdb.plug_in_gmic_qt(newImage, bilatLay, 1,0,    # 1=input only active layer
        '-v - -fx_smooth_bilateral 10,12,2,0,0')
    bilatLay.mode = LAYER_MODE_GRAIN_EXTRACT_LEGACY
# merge bilateral smoothed Grain Extract Mode layer into convolution layer
    mergeLay=pdb.gimp_image_merge_down(newImage, bilatLay, 0)
    mergeLay.name = "Temp2 Convolution"    
# intensify convo layer -- set to grain merge and duplicate selected # of times 
    intensityCnt=0
    while (intensityCnt < intensity):
        TMPLayer = 'merge'+str(intensityCnt)  
        TMPLayer = mergeLay.copy()
        TMPLayer.mode = LAYER_MODE_GRAIN_MERGE_LEGACY
        TMPLayer.opacity = 100.0 
        newImage.add_layer(TMPLayer, 0)
        pdb.gimp_item_set_visible(TMPLayer, True)
        intensityCnt=intensityCnt+1
       
# set Subject not visible &  merge all visible layers into HardSketch layer
    pdb.gimp_item_set_visible(origBG, False)
    convoLay=pdb.gimp_image_merge_visible_layers(newImage, 1)
    convoLay.mode = LAYER_MODE_GRAIN_MERGE_LEGACY
    convoLay.name = "Fake Emboss"

# Apply smoothing on convolution layer and set Subject back to visible
    #[G'MIC] Smooth [anisotropic]: -gimp_smooth_anisotropic 150,0.42,0.85,0.6,7.83,0.68,19.07,2.64,0,1,1,0,0  
    if GmicVer == 0:
        pdb.plug_in_gmic(newImage, convoLay, 1,    # 1=input only active layer
        "-v - -gimp_smooth_anisotropic 150,0.42,0.85,0.6,7.83,0.68,19.07,2.64,0,1,1,0,0")
    elif GmicVer == 1:
        pdb.plug_in_gmic_qt(newImage, convoLay, 1,0,    # 1=input only active layer
        '-v - -fx_smooth_anisotropic 150,0.42,0.85,0.6,7.83,0.68,19.07,2.64,0,1,1,0,0')

    pdb.gimp_item_set_visible(origBG, True)

# create new layer from visible as new 'subject background' for optional effect layers

    visibLay=pdb.gimp_layer_new_from_visible(newImage, newImage, 'visible')
    newImage.add_layer(visibLay, 0)
    visibLay.mode = LAYER_MODE_SOFTLIGHT_LEGACY

    origBG.name= "Original Subject"
    visibLay.name=name+suffix

    pdb.gimp_image_remove_layer(newImage, convoLay)
    pdb.gimp_item_set_visible(visibLay, True)
    pdb.gimp_item_set_visible(origBG,   True)

# scale image back to original
    pdb.gimp_image_scale(newImage, width, height)

# send new image to display    
    pdb.gimp_display_new (newImage)    
    
# Close the undo group 
    pdb.gimp_displays_flush
    pdb.gimp_context_pop()
    pdb.gimp_image_undo_group_end(inImage)    

    #return()

# This is the plugin registration function
register(
    "BA_FakeEmboss_v1b",    
    "Requires G'mic installed.\nSmaller process size = Looser detail\nLarger Process Size = Tighter Detail",     
    "Incompatible with earlier versions of G'Mic",
    "animicule@gimpchat.com", 
    "Brian Andersen ", 
    "Sept 18, 2017",
    "BA Fake Emboss...", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      #(PF_OPTION, "GmicVer", "Select your Gmic Version --->>>", 1, ("1.7.9", "2.0.0-4qt")),
      (PF_OPTION, "intensity", "Strength (low)", 0, ("Low", "Medium", "High" )),
      (PF_TOGGLE, "doublesize", "Doubles Process width (if<>0)", False),
      (PF_OPTION, "targetwidth", "Process Width     0=Subject Width", 0, ("0", "250", "500", "750", "1000", "1250", 
                "1500", "1750", "2000", "2250", "2500", "2750", "3000", "3250","3500","3750","4000" )),      
  
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    BA_FakeEmboss_v1b,
    menu="<Image>/Filters/Artistic/Brian/",
    )

main()
